@@ -9,10 +9,10 @@
 
 module CLI (runCLI) where
 
-import AST (Program (Program))
+import AST
 import Control.Exception (Exception (displayException), SomeException, handle, throw, throwIO)
 import Control.Exception.Base
-import Control.Monad (when)
+import Control.Monad (when, (>=>))
 import Data.Char (toLower, toUpper)
 import Data.Functor ((<&>))
 import Data.List (intercalate)
@@ -23,6 +23,7 @@ import Deps (saveStep)
 import Encoding (Encoding (ASCII, UNICODE))
 import Functions (buildTerm)
 import qualified Functions
+import qualified Hide as H
 import LaTeX (LatexContext (LatexContext), explainRules, programToLaTeX, rewrittensToLatex)
 import Lining (LineFormat (MULTILINE, SINGLELINE))
 import Logger
@@ -31,8 +32,9 @@ import Misc (ensuredFile)
 import qualified Misc
 import Must (Must (..))
 import Options.Applicative
-import Parser (parseProgramThrows)
+import Parser (parseExpressionThrows, parseProgramThrows)
 import Paths_phino (version)
+import Printer (printExpression')
 import qualified Printer as P
 import Rewriter (RewriteContext (RewriteContext), Rewritten (..), rewrite')
 import Sugar
@@ -47,7 +49,9 @@ data PrintProgramContext = PrintProgCtx
   { sugar :: SugarType,
     line :: LineFormat,
     xmirCtx :: XmirContext,
-    nonumber :: Bool
+    nonumber :: Bool,
+    expression :: Maybe String,
+    label :: Maybe String
   }
 
 data CmdException
@@ -113,6 +117,9 @@ data OptsRewrite = OptsRewrite
     maxCycles :: Integer,
     inPlace :: Bool,
     sequence :: Bool,
+    expression :: Maybe String,
+    label :: Maybe String,
+    hide :: [String],
     targetFile :: Maybe FilePath,
     stepsDir :: Maybe FilePath,
     inputFile :: Maybe FilePath
@@ -266,6 +273,9 @@ rewriteParser =
                 <*> optMaxCycles
                 <*> switch (long "in-place" <> help "Edit file in-place instead of printing to output")
                 <*> switch (long "sequence" <> help "Result output contains all intermediate 𝜑-programs concatenated with EOL")
+                <*> optional (strOption (long "expression" <> metavar "NAME" <> help "Name for 'phiExpression' element when rendering to LaTeX"))
+                <*> optional (strOption (long "label" <> metavar "NAME" <> help "Name for 'label' element when rendering to LaTeX"))
+                <*> many (strOption (long "hide" <> metavar "FQN" <> help "Location of object to exclude from result program after rewriting, must be a valid dispatch expression; e.g. Q.org.eolang"))
                 <*> optTarget
                 <*> optStepsDir
                 <*> argInputFile
@@ -317,6 +327,9 @@ setLogLevel' cmd =
         CmdMerge OptsMerge {logLevel} -> logLevel
    in setLogLevel level
 
+invalidCLIArguments :: String -> IO a
+invalidCLIArguments msg = throwIO (InvalidCLIArguments msg)
+
 runCLI :: [String] -> IO ()
 runCLI args = handle handler $ do
   cmd <- handleParseResult (execParserPure defaultPrefs parserInfo args)
@@ -324,14 +337,15 @@ runCLI args = handle handler $ do
   case cmd of
     CmdRewrite OptsRewrite {..} -> do
       validateOpts
+      exclude <- traverse (parseExpressionThrows >=> canBeHidden) hide
       logDebug (printf "Amount of rewriting cycles across all the rules: %d, per rule: %d" maxCycles maxDepth)
       input <- readInput inputFile
       rules' <- getRules normalize shuffle rules
       program <- parseProgram input inputFormat
       let listing = if null rules' then const input else (\prog -> P.printProgram' prog (sugarType, UNICODE, flat))
           xmirCtx = XmirContext omitListing omitComments listing
-          printProgCtx = PrintProgCtx sugarType flat xmirCtx nonumber
-      rewrittens <- rewrite' program rules' (context printProgCtx)
+          printProgCtx = PrintProgCtx sugarType flat xmirCtx nonumber expression label
+      rewrittens <- rewrite' program rules' (context printProgCtx) <&> (`H.hide` exclude)
       logDebug (printf "Printing rewritten 𝜑-program as %s" (show outputFormat))
       progs <- printRewrittens printProgCtx rewrittens
       output targetFile progs
@@ -340,13 +354,19 @@ runCLI args = handle handler $ do
         validateOpts = do
           when
             (inPlace && isNothing inputFile)
-            (throwIO (InvalidCLIArguments "--in-place requires an input file"))
+            (invalidCLIArguments "--in-place requires an input file")
           when
             (inPlace && isJust targetFile)
-            (throwIO (InvalidCLIArguments "--in-place and --target cannot be used together"))
+            (invalidCLIArguments "--in-place and --target cannot be used together")
           when
             (nonumber && outputFormat /= LATEX)
-            (throwIO (InvalidCLIArguments "The --nonumber option can stay together with --output=latex only"))
+            (invalidCLIArguments "The --nonumber option can stay together with --output=latex only")
+          when
+            (isJust expression && outputFormat /= LATEX)
+            (invalidCLIArguments "The --expression option can stay together with --output=latex only")
+          when
+            (isJust label && outputFormat /= LATEX)
+            (invalidCLIArguments "The --label option can stay together with --output=latex only")
           validateMaxDepth maxDepth
           validateMaxCycles maxCycles
           validateMust must
@@ -376,8 +396,20 @@ runCLI args = handle handler $ do
             (saveStep stepsDir (ioToExtension outputFormat) (printProgram outputFormat ctx))
         printRewrittens :: PrintProgramContext -> [Rewritten] -> IO String
         printRewrittens ctx rewrittens
-          | outputFormat == LATEX = pure (rewrittensToLatex rewrittens (LatexContext sugarType flat nonumber))
+          | outputFormat == LATEX = pure (rewrittensToLatex rewrittens (LatexContext sugarType flat nonumber expression label))
           | otherwise = mapM (printProgram outputFormat ctx . program) rewrittens <&> intercalate "\n"
+        canBeHidden :: Expression -> IO Expression
+        canBeHidden expr = canBeHidden' expr expr
+          where
+            canBeHidden' :: Expression -> Expression -> IO Expression
+            canBeHidden' exp@(ExDispatch ExGlobal _) _ = pure exp
+            canBeHidden' exp@(ExDispatch expr attr) full = canBeHidden' expr full >> pure exp
+            canBeHidden' _ full =
+              invalidCLIArguments
+                ( printf
+                    "Only dispatch expression started with Φ (or Q) can be used in --hide, but given: %s"
+                    (printExpression' full P.logPrintConfig)
+                )
     CmdDataize OptsDataize {..} -> do
       validateOpts
       input <- readInput inputFile
@@ -397,7 +429,7 @@ runCLI args = handle handler $ do
             maxCycles
             depthSensitive
             buildTerm
-            (saveStep stepsDir (ioToExtension PHI) (printProgram PHI (PrintProgCtx sugarType flat defaultXmirContext False)))
+            (saveStep stepsDir (ioToExtension PHI) (printProgram PHI (PrintProgCtx sugarType flat defaultXmirContext False Nothing Nothing)))
     CmdExplain OptsExplain {..} -> do
       validateOpts
       rules' <- getRules normalize shuffle rules
@@ -416,7 +448,7 @@ runCLI args = handle handler $ do
       prog <- merge progs
       let listing = const (P.printProgram' prog (sugarType, UNICODE, flat))
           xmirCtx = XmirContext omitListing omitComments listing
-          printProgCtx = PrintProgCtx sugarType flat xmirCtx False
+          printProgCtx = PrintProgCtx sugarType flat xmirCtx False Nothing Nothing
       prog' <- printProgram outputFormat printProgCtx prog
       output targetFile prog'
       where
@@ -431,15 +463,15 @@ runCLI args = handle handler $ do
     validateXmirOptions outputFormat omitListing omitComments = do
       when
         (outputFormat /= XMIR && omitListing)
-        (throwIO (InvalidCLIArguments "--omit-listing can be used only with --output-format=xmir"))
+        (invalidCLIArguments "--omit-listing can be used only with --output-format=xmir")
       when
         (outputFormat /= XMIR && omitComments)
-        (throwIO (InvalidCLIArguments "--omit-comments can be used only with --output-format=xmir"))
+        (invalidCLIArguments "--omit-comments can be used only with --output-format=xmir")
     validateIntArgument :: Integer -> (Integer -> Bool) -> String -> IO ()
     validateIntArgument num cmp msg =
       when
         (cmp num)
-        (throwIO (InvalidCLIArguments msg))
+        (invalidCLIArguments msg)
     validateMaxDepth :: Integer -> IO ()
     validateMaxDepth depth = validateIntArgument depth (<= 0) "--max-depth must be positive"
     validateMaxCycles :: Integer -> IO ()
@@ -453,10 +485,8 @@ runCLI args = handle handler $ do
       case (minVal, maxVal) of
         (Just min, Just max)
           | min > max ->
-              throwIO
-                ( InvalidCLIArguments
-                    (printf "--must range invalid: minimum (%d) is greater than maximum (%d)" min max)
-                )
+              invalidCLIArguments
+                (printf "--must range invalid: minimum (%d) is greater than maximum (%d)" min max)
         _ -> pure ()
     readInput :: Maybe FilePath -> IO String
     readInput inputFile' = case inputFile' of
@@ -474,7 +504,7 @@ runCLI args = handle handler $ do
     printProgram :: IOFormat -> PrintProgramContext -> Program -> IO String
     printProgram PHI PrintProgCtx {..} prog = pure (P.printProgram' prog (sugar, UNICODE, line))
     printProgram XMIR PrintProgCtx {..} prog = programToXMIR prog xmirCtx <&> printXMIR
-    printProgram LATEX PrintProgCtx {..} prog = pure (programToLaTeX prog (LatexContext sugar line nonumber))
+    printProgram LATEX PrintProgCtx {..} prog = pure (programToLaTeX prog (LatexContext sugar line nonumber expression label))
     getRules :: Bool -> Bool -> [FilePath] -> IO [Y.Rule]
     getRules normalize shuffle rules = do
       ordered <-
