@@ -19,7 +19,7 @@ class ToASCII a where
 
 instance ToASCII PROGRAM where
   toASCII PR_SALTY {..} = PR_SALTY Q ARROW' (toASCII expr)
-  toASCII PR_SWEET {..} = PR_SWEET (toASCII expr)
+  toASCII PR_SWEET {..} = PR_SWEET lcb (toASCII expr) rcb
 
 instance ToASCII EXPRESSION where
   toASCII EX_GLOBAL {..} = EX_GLOBAL Q
@@ -31,14 +31,18 @@ instance ToASCII EXPRESSION where
   toASCII EX_DISPATCH {..} = EX_DISPATCH (toASCII expr) (toASCII attr)
   toASCII EX_APPLICATION {..} = EX_APPLICATION (toASCII expr) eol tab (toASCII bindings) eol' tab'
   toASCII EX_APPLICATION' {..} = EX_APPLICATION' (toASCII expr) eol tab (toASCII args) eol' tab'
+  toASCII EX_META {..} = EX_META (MT_EXPRESSION' (rest meta))
+  toASCII EX_META_TAIL {..} = EX_META_TAIL (toASCII expr) (MT_TAIL (rest meta))
   toASCII expr = expr
 
 instance ToASCII BINDING where
   toASCII BI_PAIR {..} = BI_PAIR (toASCII pair) (toASCII bindings) tab
+  toASCII BI_META {..} = BI_META (MT_BINDING' (rest meta)) (toASCII bindings) tab
   toASCII bd = bd
 
 instance ToASCII BINDINGS where
   toASCII BDS_PAIR {..} = BDS_PAIR eol tab (toASCII pair) (toASCII bindings)
+  toASCII BDS_META {..} = BDS_META eol tab (MT_BINDING' (rest meta)) (toASCII bindings)
   toASCII bds = bds
 
 instance ToASCII APP_ARG where
@@ -53,10 +57,13 @@ instance ToASCII PAIR where
   toASCII PA_VOID {..} = PA_VOID (toASCII attr) ARROW' QUESTION
   toASCII PA_LAMBDA {..} = PA_LAMBDA' func
   toASCII PA_DELTA {..} = PA_DELTA' bytes
+  toASCII PA_META_LAMBDA {..} = PA_META_LAMBDA' meta
+  toASCII PA_META_DELTA {..} = PA_META_DELTA' (MT_BYTES' (rest meta))
   toASCII pair = pair
 
 instance ToASCII ATTRIBUTE where
   toASCII AT_ALPHA {..} = AT_ALPHA ALPHA' idx
   toASCII AT_PHI {..} = AT_PHI AT
   toASCII AT_RHO {..} = AT_RHO CARET
+  toASCII AT_META {..} = AT_META (MT_ATTRIBUTE' (rest meta))
   toASCII attr = attr
