@@ -23,7 +23,9 @@ import qualified Yaml as Y
 data LatexContext = LatexContext
   { sugar :: SugarType,
     line :: LineFormat,
-    nonumber :: Bool
+    nonumber :: Bool,
+    expression :: Maybe String,
+    label :: Maybe String
   }
 
 renderToLatex :: Program -> LatexContext -> String
@@ -34,10 +36,16 @@ phiquation LatexContext {nonumber = True} = "phiquation*"
 phiquation LatexContext {nonumber = False} = "phiquation"
 
 rewrittensToLatex :: [Rewritten] -> LatexContext -> String
-rewrittensToLatex rewrittens ctx =
+rewrittensToLatex rewrittens ctx@LatexContext {..} =
   let equation = phiquation ctx
    in concat
         [ printf "\\begin{%s}\n" equation,
+          case label of
+            Just lbl -> printf "\\label{%s}\n" lbl
+            _ -> "",
+          case expression of
+            Just expr -> printf "\\phiExpression{%s} " expr
+            _ -> "",
           intercalate
             "\n  \\leadsto "
             ( map
@@ -69,11 +77,14 @@ programToLaTeX prog ctx =
           "}"
         ]
 
+piped :: String -> String
+piped str = "|" <> str <> "|"
+
 class ToLaTeX a where
   toLaTeX :: a -> a
 
 instance ToLaTeX PROGRAM where
-  toLaTeX PR_SWEET {..} = PR_SWEET (toLaTeX expr)
+  toLaTeX PR_SWEET {..} = PR_SWEET BIG_LCB (toLaTeX expr) BIG_RCB
   toLaTeX PR_SALTY {..} = PR_SALTY global arrow (toLaTeX expr)
 
 instance ToLaTeX EXPRESSION where
@@ -81,14 +92,11 @@ instance ToLaTeX EXPRESSION where
   toLaTeX EX_FORMATION {..} = EX_FORMATION lsb eol tab (toLaTeX binding) eol' tab' rsb
   toLaTeX EX_APPLICATION {..} = EX_APPLICATION (toLaTeX expr) eol tab (toLaTeX bindings) eol' tab'
   toLaTeX EX_APPLICATION' {..} = EX_APPLICATION' (toLaTeX expr) eol tab (toLaTeX args) eol' tab'
-  toLaTeX EX_DISPATCH{..} = EX_DISPATCH (toLaTeX expr) (toLaTeX attr)
+  toLaTeX EX_DISPATCH {..} = EX_DISPATCH (toLaTeX expr) (toLaTeX attr)
   toLaTeX expr = expr
 
 instance ToLaTeX ATTRIBUTE where
   toLaTeX AT_LABEL {..} = AT_LABEL (piped (toLaTeX label))
-    where
-      piped :: String -> String
-      piped str = "|" <> str <> "|"
   toLaTeX attr = attr
 
 instance ToLaTeX BINDING where
@@ -101,7 +109,8 @@ instance ToLaTeX BINDINGS where
 
 instance ToLaTeX PAIR where
   toLaTeX PA_DELTA {..} = PA_DELTA' bytes
-  toLaTeX PA_LAMBDA {..} = PA_LAMBDA' func
+  toLaTeX PA_LAMBDA {..} = PA_LAMBDA' (piped (toLaTeX func))
+  toLaTeX PA_LAMBDA' {..} = PA_LAMBDA' (piped (toLaTeX func))
   toLaTeX PA_VOID {..} = PA_VOID (toLaTeX attr) arrow void
   toLaTeX PA_TAU {..} = PA_TAU (toLaTeX attr) arrow (toLaTeX expr)
   toLaTeX pair = pair
@@ -122,6 +131,7 @@ instance ToLaTeX String where
         '$' -> "\\char36{}" <> escapeUnprintedChars rest
         '@' -> "\\char64{}" <> escapeUnprintedChars rest
         '^' -> "\\char94{}" <> escapeUnprintedChars rest
+        '_' -> "\\char95{}" <> escapeUnprintedChars rest
         _ -> ch : escapeUnprintedChars rest
 
 -- @todo #114:30min Implement LaTeX conversion for rules.
