@@ -20,7 +20,7 @@ class ToSingleLine a where
 
 instance ToSingleLine PROGRAM where
   toSingleLine PR_SALTY {..} = PR_SALTY global arrow (toSingleLine expr)
-  toSingleLine PR_SWEET {..} = PR_SWEET (toSingleLine expr)
+  toSingleLine PR_SWEET {..} = PR_SWEET lcb (toSingleLine expr) rcb
 
 instance ToSingleLine EXPRESSION where
   toSingleLine EX_FORMATION {lsb, binding = bd@BI_EMPTY {..}, rsb} = EX_FORMATION lsb NO_EOL NO_TAB bd NO_EOL NO_TAB rsb
@@ -32,10 +32,12 @@ instance ToSingleLine EXPRESSION where
 
 instance ToSingleLine BINDING where
   toSingleLine BI_PAIR {..} = BI_PAIR (toSingleLine pair) (toSingleLine bindings) TAB'
+  toSingleLine BI_META {..} = BI_META meta (toSingleLine bindings) TAB'
   toSingleLine bd = bd
 
 instance ToSingleLine BINDINGS where
   toSingleLine BDS_PAIR {..} = BDS_PAIR NO_EOL TAB' (toSingleLine pair) (toSingleLine bindings)
+  toSingleLine BDS_META {..} = BDS_META NO_EOL TAB' meta (toSingleLine bindings)
   toSingleLine bds = bds
 
 instance ToSingleLine PAIR where
