@@ -25,6 +25,14 @@ instance Render Double where
 instance Render Char where
   render ch = [ch]
 
+instance Render LCB where
+  render LCB = "{"
+  render BIG_LCB = "\\Big\\{"
+
+instance Render RCB where
+  render RCB = "}"
+  render BIG_RCB = "\\Big\\}"
+
 instance Render LSB where
   render LSB = "⟦"
   render LSB' = "[["
@@ -90,6 +98,18 @@ instance Render BYTES where
   render (BT_ONE bte) = render bte <> "-"
   render (BT_MANY bts) = intercalate "-" bts
 
+instance Render META where
+  render MT_EXPRESSION {..} = '𝑒' : rest
+  render MT_EXPRESSION' {..} = "!e" <> rest
+  render MT_ATTRIBUTE {..} = '𝜏' : rest
+  render MT_ATTRIBUTE' {..} = "!a" <> rest
+  render MT_BINDING {..} = '𝐵' : rest
+  render MT_BINDING' {..} = "!B" <> rest
+  render MT_BYTES {..} = 'δ' : rest
+  render MT_BYTES' {..} = "!d" <> rest
+  render MT_TAIL {..} = "!t" <> rest
+  render MT_FUNCTION {..} = "!F" <> rest
+
 instance Render ALPHA where
   render ALPHA = "α"
   render ALPHA' = "~"
@@ -100,7 +120,7 @@ instance Render TAB where
   render NO_TAB = ""
 
 instance Render PROGRAM where
-  render PR_SWEET {..} = "{" <> render expr <> "}"
+  render PR_SWEET {..} = render lcb <> render expr <> render rcb
   render PR_SALTY {..} = render global <> render SPACE <> render arrow <> render SPACE <> render expr
 
 instance Render PAIR where
@@ -110,14 +130,20 @@ instance Render PAIR where
   render PA_VOID {..} = render attr <> render SPACE <> render arrow <> render SPACE <> render void
   render PA_DELTA {..} = render DELTA <> render SPACE <> render DASHED_ARROW <> render SPACE <> render bytes
   render PA_DELTA' {..} = "D> " <> render bytes
+  render PA_META_LAMBDA {..} = render LAMBDA <> render SPACE <> render DASHED_ARROW <> render SPACE <> render meta
+  render PA_META_LAMBDA' {..} = render "L> " <> render meta
+  render PA_META_DELTA {..} = render DELTA <> render SPACE <> render DASHED_ARROW <> render SPACE <> render meta
+  render PA_META_DELTA' {..} = render "D> " <> render meta
 
 instance Render BINDINGS where
   render BDS_EMPTY {..} = ""
   render BDS_PAIR {..} = render COMMA <> render eol <> render tab <> render pair <> render bindings
+  render BDS_META {..} = render COMMA <> render eol <> render tab <> render meta <> render bindings
 
 instance Render BINDING where
-  render BI_PAIR {..} = render pair <> render bindings
   render BI_EMPTY {..} = ""
+  render BI_PAIR {..} = render pair <> render bindings
+  render BI_META {..} = render meta <> render bindings
 
 instance Render APP_ARG where
   render APP_ARG {..} = render expr <> render args
@@ -142,6 +168,8 @@ instance Render EXPRESSION where
   render EX_APPLICATION' {..} = render expr <> "(" <> render eol <> render tab <> render args <> render eol' <> render tab' <> ")"
   render EX_STRING {..} = '"' : render str <> "\""
   render EX_NUMBER {..} = either show show num
+  render EX_META {..} = render meta
+  render EX_META_TAIL {..} = render expr <> " * " <> render meta
 
 instance Render ATTRIBUTE where
   render AT_LABEL {..} = label
@@ -150,3 +178,4 @@ instance Render ATTRIBUTE where
   render AT_PHI {..} = render phi
   render AT_LAMBDA {..} = render lambda
   render AT_DELTA {..} = render delta
+  render AT_META {..} = render meta
