@@ -181,6 +181,24 @@ spec = do
             ["rewrite", "--omit-comments", "--output=phi"]
             ["--omit-comments"]
 
+      it "with --expression and --output != latex" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--expression=foo", "--output=phi"]
+            ["--expression option can stay together with --output=latex only"]
+
+      it "with --label and --output != latex" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--label=foo", "--output=phi"]
+            ["--label option can stay together with --output=latex only"]
+
+      it "with wrong --hide option" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--hide=Q.x(Q.y)"]
+            ["[ERROR]: Invalid set of arguments: Only dispatch expression", "but given: Φ.x( Φ.y )"]
+
     it "saves steps to dir with --steps-dir" $ do
       let dir = "test-steps-temp"
       dirExists <- doesDirectoryExist dir
@@ -253,20 +271,21 @@ spec = do
           ["<?xml version=\"1.0\" encoding=\"UTF-8\"?>", "<object", "  <o base=\"Φ.y\" name=\"x\"/>"]
 
     it "rewrites as LaTeX" $
-      withStdin "Q -> [[ x -> QQ.z(y -> 5), q -> T, w -> $, ^ -> Q, @ -> 1, y -> \"H$@^M\"]]" $
+      withStdin "Q -> [[ x_o -> QQ.z(y -> 5), q$ -> T, w -> $, ^ -> Q, @ -> 1, y -> \"H$@^M\", L> Fu_nc ]]" $
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet"]
           [ "\\begin{phiquation}",
-            "{[[",
-            "  |x| -> QQ.|z|(",
-            "    |y| -> 5",
-            "  ),",
-            "  |q| -> T,",
-            "  |w| -> $,",
-            "  ^ -> Q,",
-            "  @ -> 1,",
-            "  |y| -> \"H$@^M\"",
-            "]]}",
+            "\\Big\\{[[\n",
+            "  |x\\char95{}o| -> QQ.|z|(\n",
+            "    |y| -> 5\n",
+            "  ),\n",
+            "  |q\\char36{}| -> T,\n",
+            "  |w| -> $,\n",
+            "  ^ -> Q,\n",
+            "  @ -> 1,\n",
+            "  |y| -> \"H$@^M\",\n",
+            "  L> |Fu\\char95{}nc|\n",
+            "]]\\Big\\}",
             "\\end{phiquation}"
           ]
 
@@ -275,10 +294,28 @@ spec = do
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet", "--nonumber", "--flat"]
           [ "\\begin{phiquation*}",
-            "{[[ |x| -> 5 ]]}",
+            "\\Big\\{[[ |x| -> 5 ]]\\Big\\}",
             "\\end{phiquation*}"
           ]
 
+    it "rewrite as LaTeX with expression name" $
+      withStdin "Q -> [[ x -> 5 ]]" $
+        testCLISucceeded
+          ["rewrite", "--output=latex", "--sweet", "--flat", "--expression=foo"]
+          [ "\\begin{phiquation}",
+            "\\phiExpression{foo} \\Big\\{[[ |x| -> 5 ]]\\Big\\}",
+            "\\end{phiquation}"
+          ]
+
+    it "rewrite as LaTeX with label name" $
+      withStdin "Q -> [[ x -> 5 ]]" $
+        testCLISucceeded
+          ["rewrite", "--output=latex", "--sweet", "--flat", "--label=foo"]
+          [ "\\begin{phiquation}\n\\label{foo}\n",
+            "\\Big\\{[[ |x| -> 5 ]]\\Big\\}\n",
+            "\\end{phiquation}"
+          ]
+
     it "rewrites with XMIR as input" $
       withStdin "<object><o name=\"app\"><o name=\"x\" base=\"Φ.number\"/></o></object>" $
         testCLISucceeded
@@ -338,9 +375,9 @@ spec = do
           ]
           [ unlines
               [ "\\begin{phiquation}",
-                "{[[ |x| -> \"foo\" ]]} \\leadsto_{\\nameref{r:first}}",
-                "  \\leadsto {Q.|x|( |y| -> \"foo\" )} \\leadsto_{\\nameref{r:second}}",
-                "  \\leadsto {[[ |x| -> \"foo\" ]]}",
+                "\\Big\\{[[ |x| -> \"foo\" ]]\\Big\\} \\leadsto_{\\nameref{r:first}}",
+                "  \\leadsto \\Big\\{Q.|x|( |y| -> \"foo\" )\\Big\\} \\leadsto_{\\nameref{r:second}}",
+                "  \\leadsto \\Big\\{[[ |x| -> \"foo\" ]]\\Big\\}",
                 "\\end{phiquation}"
               ]
           ]
@@ -444,6 +481,18 @@ spec = do
               ]
           ]
 
+    it "hides default package" $
+      withStdin "{[[ org -> [[ eolang -> [[ number -> [[]] ]]]], x -> 42 ]]}" $
+        testCLISucceeded
+          ["rewrite", "--sweet", "--flat", "--hide=Q.org"]
+          ["{⟦ x ↦ 42 ⟧}"]
+
+    it "hides several FQNs" $
+      withStdin "{[[ org -> [[ eolang -> Q.x, yegor256 -> Q.y ]], x -> 42 ]]}" $
+        testCLISucceeded
+          ["rewrite", "--sweet", "--flat", "--hide=Q.org.eolang", "--hide=Q.org.yegor256"]
+          ["{⟦ org ↦ ⟦⟧, x ↦ 42 ⟧}"]
+
     it "prints in line with --flat" $
       withStdin "Q -> [[ x -> 5, y -> \"hey\", z -> [[ w -> [[ ]] ]] ]]" $
         testCLISucceeded
